import csv
import argparse

PREROLL_TIME = 5 

def duplicate_and_insert_first_row(file_path, output_path, rate, add_steps):
    """
    Reads a CSV file, duplicates the first row with adjusted timestamps,
    inserts them before the original first row, and saves the modified file.
    
    :param file_path: Path to the input CSV file
    :param output_path: Path to save the modified CSV file
    :param rate: Rate in Hz, converted to time step in nanoseconds
    """
    time_step_ns = int((1.0 / rate) * 1e9)  # Convert rate to nanoseconds
    total_time_ns = PREROLL_TIME * 1e9  # e.g. 10 seconds in nanoseconds
    num_duplicates = int(total_time_ns / time_step_ns)
    
    with open(file_path, 'r', newline='') as f:
        reader = csv.reader(f)
        rows = list(reader)
    
    if not rows:
        print("Error: CSV file is empty.")
        return
    
    header = rows[0]
    if len(rows) < 2:
        print("Error: CSV file does not contain enough data rows.")
        return
    
    first_row = rows[1]
    timestamp_index = header.index("timestamp(ns)")
    first_timestamp = int(first_row[timestamp_index])
    
    new_rows = []
    batch = 1
    idx = 1
    for i in range(num_duplicates, 0, -1):
        new_row = first_row[:]
        new_row[timestamp_index] = str(first_timestamp - (i * time_step_ns))
        if add_steps:
            new_row[0] = idx 
            new_row[2] = batch
            new_row[3] = 0.0
            new_row[4] = 0.0
            new_row[5] = -9.8
            new_row[6] = 0.0
            new_row[7] = 0.0
            new_row[8] = 0.0
            idx = idx + 1
            if idx % 32 == 0:
                batch = batch + 1

        new_rows.append(new_row)


    
    batch = batch + 1
    for row in rows[1:]:
        row[0] = idx
        row[2] = batch
        idx = idx + 1
        if idx % 32 == 0:
            batch = batch + 1

    
    with open(output_path, 'w', newline='') as f:
        writer = csv.writer(f)
        writer.writerow(header)
        writer.writerows(new_rows + rows[1:])
    
    print(f"Modified file saved as: {output_path}")


def main():
    parser = argparse.ArgumentParser(description="Duplicate and insert first row with adjusted timestamps.")
    parser.add_argument("input_file", type=str, help="Path to the input CSV file")
    parser.add_argument("output_file", type=str, help="Path to save the modified CSV file")
    parser.add_argument("rate", type=float, help="Rate in Hz to determine time step in nanoseconds")
    parser.add_argument("add", type=int, help="add time steps")
    args = parser.parse_args()
    
    duplicate_and_insert_first_row(args.input_file, args.output_file, args.rate, args.add)


if __name__ == "__main__":
    main()
